@@ -65,15 +65,33 @@ object CometConf extends ShimCometConf {
     .booleanConf
     .createWithDefault(sys.env.getOrElse("ENABLE_COMET", "true").toBoolean)
 
-  val COMET_SCAN_ENABLED: ConfigEntry[Boolean] = conf("spark.comet.scan.enabled")
+  val COMET_NATIVE_SCAN_ENABLED: ConfigEntry[Boolean] = conf("spark.comet.scan.enabled")
     .doc(
-      "Whether to enable Comet scan. When this is turned on, Spark will use Comet to read " +
-        "Parquet data source. Note that to enable native vectorized execution, both this " +
-        "config and 'spark.comet.exec.enabled' need to be enabled. By default, this config " +
-        "is true.")
+      "Whether to enable native scans. When this is turned on, Spark will use Comet to " +
+        "read supported data sources (currently only Parquet is supported natively). Note " +
+        "that to enable native vectorized execution, both this config and " +
+        "'spark.comet.exec.enabled' need to be enabled. By default, this config is true.")
     .booleanConf
     .createWithDefault(true)
 
+  val COMET_CONVERT_FROM_PARQUET_ENABLED: ConfigEntry[Boolean] =
+    conf("spark.comet.convert.parquet.enabled")
+      .doc(
+        "When enabled, data from Spark (non-native) Parquet v1 and v2 scans will be converted to " +
+          "Arrow format. Note that to enable native vectorized execution, both this config and " +
+          "'spark.comet.exec.enabled' need to be enabled.")
+      .booleanConf
+      .createWithDefault(false)
+
+  val COMET_CONVERT_FROM_JSON_ENABLED: ConfigEntry[Boolean] =
+    conf("spark.comet.convert.json.enabled")
+      .doc(
+        "When enabled, data from Spark (non-native) JSON v1 and v2 scans will be converted to " +
+          "Arrow format. Note that to enable native vectorized execution, both this config and " +
+          "'spark.comet.exec.enabled' need to be enabled.")
+      .booleanConf
+      .createWithDefault(false)
+
   val COMET_EXEC_ENABLED: ConfigEntry[Boolean] = conf(s"$COMET_EXEC_CONFIG_PREFIX.enabled")
     .doc(
       "Whether to enable Comet native vectorized execution for Spark. This controls whether " +
@@ -81,9 +99,48 @@ object CometConf extends ShimCometConf {
         "native space. Note: each operator is associated with a separate config in the " +
         "format of 'spark.comet.exec.<operator_name>.enabled' at the moment, and both the " +
         "config and this need to be turned on, in order for the operator to be executed in " +
-        "native. By default, this config is false.")
+        "native. By default, this config is true.")
     .booleanConf
-    .createWithDefault(false)
+    .createWithDefault(true)
+
+  val COMET_EXEC_PROJECT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("project", defaultValue = true)
+  val COMET_EXEC_FILTER_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("filter", defaultValue = true)
+  val COMET_EXEC_SORT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("sort", defaultValue = true)
+  val COMET_EXEC_LOCAL_LIMIT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("localLimit", defaultValue = true)
+  val COMET_EXEC_GLOBAL_LIMIT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("globalLimit", defaultValue = true)
+  val COMET_EXEC_BROADCAST_HASH_JOIN_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("broadcastHashJoin", defaultValue = true)
+  val COMET_EXEC_BROADCAST_EXCHANGE_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("broadcastExchange", defaultValue = true)
+  val COMET_EXEC_HASH_JOIN_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("hashJoin", defaultValue = true)
+  val COMET_EXEC_SORT_MERGE_JOIN_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("sortMergeJoin", defaultValue = true)
+  val COMET_EXEC_AGGREGATE_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("aggregate", defaultValue = true)
+  val COMET_EXEC_COLLECT_LIMIT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("collectLimit", defaultValue = true)
+  val COMET_EXEC_COALESCE_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("coalesce", defaultValue = true)
+  val COMET_EXEC_UNION_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("union", defaultValue = true)
+  val COMET_EXEC_EXPAND_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("expand", defaultValue = true)
+  val COMET_EXEC_WINDOW_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("window", defaultValue = true)
+  val COMET_EXEC_TAKE_ORDERED_AND_PROJECT_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig("takeOrderedAndProject", defaultValue = true)
+
+  val COMET_EXPR_STDDEV_ENABLED: ConfigEntry[Boolean] =
+    createExecEnabledConfig(
+      "stddev",
+      defaultValue = true,
+      notes = Some("stddev is slower than Spark's implementation"))
 
   val COMET_MEMORY_OVERHEAD: OptionalConfigEntry[Long] = conf("spark.comet.memoryOverhead")
     .doc(
@@ -114,15 +171,6 @@ object CometConf extends ShimCometConf {
       "Ensure that Comet memory overhead min is a long greater than or equal to 0")
     .createWithDefault(384)
 
-  val COMET_EXEC_ALL_OPERATOR_ENABLED: ConfigEntry[Boolean] = conf(
-    s"$COMET_EXEC_CONFIG_PREFIX.all.enabled")
-    .doc(
-      "Whether to enable all Comet operators. By default, this config is false. Note that " +
-        "this config precedes all separate config 'spark.comet.exec.<operator_name>.enabled'. " +
-        "That being said, if this config is enabled, separate configs are ignored.")
-    .booleanConf
-    .createWithDefault(false)
-
   val COMET_EXEC_SHUFFLE_ENABLED: ConfigEntry[Boolean] =
     conf(s"$COMET_EXEC_CONFIG_PREFIX.shuffle.enabled")
       .doc(
@@ -146,17 +194,6 @@ object CometConf extends ShimCometConf {
     .checkValues(Set("native", "jvm", "auto"))
     .createWithDefault("jvm")
 
-  val COMET_SHUFFLE_ENFORCE_MODE_ENABLED: ConfigEntry[Boolean] =
-    conf("spark.comet.shuffle.enforceMode.enabled")
-      .doc(
-        "Comet shuffle doesn't support Spark AQE coalesce partitions. If AQE coalesce " +
-          "partitions is enabled, Comet shuffle won't be triggered even enabled. This config " +
-          "is used to enforce Comet to trigger shuffle even if AQE coalesce partitions is " +
-          "enabled. This is for testing purpose only.")
-      .internal()
-      .booleanConf
-      .createWithDefault(false)
-
   val COMET_EXEC_BROADCAST_FORCE_ENABLED: ConfigEntry[Boolean] =
     conf(s"$COMET_EXEC_CONFIG_PREFIX.broadcast.enabled")
       .doc(
@@ -222,8 +259,8 @@ object CometConf extends ShimCometConf {
     conf("spark.comet.columnar.shuffle.memorySize")
       .doc(
         "The optional maximum size of the memory used for Comet columnar shuffle, in MiB. " +
-          "Note that this config is only used when `spark.comet.columnar.shuffle.enabled` is " +
-          "true. Once allocated memory size reaches this config, the current batch will be " +
+          "Note that this config is only used when `spark.comet.exec.shuffle.mode` is " +
+          "`jvm`. Once allocated memory size reaches this config, the current batch will be " +
           "flushed to disk immediately. If this is not configured, Comet will use " +
           "`spark.comet.shuffle.memory.factor` * `spark.comet.memoryOverhead` as " +
           "shuffle memory size. If final calculated value is larger than Comet memory " +
@@ -259,7 +296,7 @@ object CometConf extends ShimCometConf {
       "prefer dictionary encoding when shuffling the column. If the ratio is higher than " +
       "this config, dictionary encoding will be used on shuffling string column. This config " +
       "is effective if it is higher than 1.0. By default, this config is 10.0. Note that this " +
-      "config is only used when 'spark.comet.columnar.shuffle.enabled' is true.")
+      "config is only used when `spark.comet.exec.shuffle.mode` is `jvm`.")
     .doubleConf
     .createWithDefault(10.0)
 
@@ -282,6 +319,15 @@ object CometConf extends ShimCometConf {
       .booleanConf
       .createWithDefault(false)
 
+  val COMET_EXPLAIN_NATIVE_ENABLED: ConfigEntry[Boolean] =
+    conf("spark.comet.explain.native.enabled")
+      .doc(
+        "When this setting is enabled, Comet will provide a tree representation of " +
+          "the native query plan before execution and again after execution, with " +
+          "metrics.")
+      .booleanConf
+      .createWithDefault(false)
+
   val COMET_EXPLAIN_FALLBACK_ENABLED: ConfigEntry[Boolean] =
     conf("spark.comet.explainFallback.enabled")
       .doc(
@@ -291,6 +337,22 @@ object CometConf extends ShimCometConf {
       .booleanConf
       .createWithDefault(false)
 
+  val COMET_WORKER_THREADS: ConfigEntry[Int] =
+    conf("spark.comet.workerThreads")
+      .internal()
+      .doc("The number of worker threads used for Comet native execution. " +
+        "By default, this config is 4.")
+      .intConf
+      .createWithDefault(4)
+
+  val COMET_BLOCKING_THREADS: ConfigEntry[Int] =
+    conf("spark.comet.blockingThreads")
+      .internal()
+      .doc("The number of blocking threads used for Comet native execution. " +
+        "By default, this config is 10.")
+      .intConf
+      .createWithDefault(10)
+
   val COMET_BATCH_SIZE: ConfigEntry[Int] = conf("spark.comet.batchSize")
     .doc("The columnar batch size, i.e., the maximum number of rows that a batch can contain.")
     .intConf
@@ -374,20 +436,21 @@ object CometConf extends ShimCometConf {
     .booleanConf
     .createWithDefault(COMET_SCHEMA_EVOLUTION_ENABLED_DEFAULT)
 
-  val COMET_ROW_TO_COLUMNAR_ENABLED: ConfigEntry[Boolean] =
-    conf("spark.comet.rowToColumnar.enabled")
+  val COMET_SPARK_TO_ARROW_ENABLED: ConfigEntry[Boolean] =
+    conf("spark.comet.sparkToColumnar.enabled")
       .internal()
-      .doc("Whether to enable row to columnar conversion in Comet. When this is turned on, " +
-        "Comet will convert row-based operators in " +
-        "`spark.comet.rowToColumnar.supportedOperatorList` into columnar based before processing.")
+      .doc("Whether to enable Spark to Arrow columnar conversion. When this is turned on, " +
+        "Comet will convert operators in " +
+        "`spark.comet.sparkToColumnar.supportedOperatorList` into Arrow columnar format before " +
+        "processing.")
       .booleanConf
       .createWithDefault(false)
 
-  val COMET_ROW_TO_COLUMNAR_SUPPORTED_OPERATOR_LIST: ConfigEntry[Seq[String]] =
-    conf("spark.comet.rowToColumnar.supportedOperatorList")
+  val COMET_SPARK_TO_ARROW_SUPPORTED_OPERATOR_LIST: ConfigEntry[Seq[String]] =
+    conf("spark.comet.sparkToColumnar.supportedOperatorList")
       .doc(
-        "A comma-separated list of row-based operators that will be converted to columnar " +
-          "format when 'spark.comet.rowToColumnar.enabled' is true")
+        "A comma-separated list of operators that will be converted to Arrow columnar " +
+          "format when 'spark.comet.sparkToColumnar.enabled' is true")
       .stringConf
       .toSequence
       .createWithDefault(Seq("Range,InMemoryTableScan"))
@@ -401,6 +464,14 @@ object CometConf extends ShimCometConf {
     .booleanConf
     .createWithDefault(COMET_ANSI_MODE_ENABLED_DEFAULT)
 
+  val COMET_CASE_CONVERSION_ENABLED: ConfigEntry[Boolean] =
+    conf("spark.comet.caseConversion.enabled")
+      .doc(
+        "Java uses locale-specific rules when converting strings to upper or lower case and " +
+          "Rust does not, so we disable upper and lower by default.")
+      .booleanConf
+      .createWithDefault(false)
+
   val COMET_CAST_ALLOW_INCOMPATIBLE: ConfigEntry[Boolean] =
     conf("spark.comet.cast.allowIncompatible")
       .doc(
@@ -410,6 +481,27 @@ object CometConf extends ShimCometConf {
       .booleanConf
       .createWithDefault(false)
 
+  val COMET_REGEXP_ALLOW_INCOMPATIBLE: ConfigEntry[Boolean] =
+    conf("spark.comet.regexp.allowIncompatible")
+      .doc("Comet is not currently fully compatible with Spark for all regular expressions. " +
+        "Set this config to true to allow them anyway using Rust's regular expression engine. " +
+        "See compatibility guide for more information.")
+      .booleanConf
+      .createWithDefault(false)
+
+  /** Create a config to enable a specific operator */
+  private def createExecEnabledConfig(
+      exec: String,
+      defaultValue: Boolean,
+      notes: Option[String] = None): ConfigEntry[Boolean] = {
+    conf(s"$COMET_EXEC_CONFIG_PREFIX.$exec.enabled")
+      .doc(
+        s"Whether to enable $exec by default." + notes
+          .map(s => s" $s.")
+          .getOrElse(""))
+      .booleanConf
+      .createWithDefault(defaultValue)
+  }
 }
 
 object ConfigHelpers {
