@@ -53,6 +53,7 @@ use crate::{
     jvm_bridge::{jni_new_global_ref, JVMClasses},
 };
 use datafusion_comet_proto::spark_operator::Operator;
+use datafusion_common::ScalarValue;
 use futures::stream::StreamExt;
 use jni::{
     objects::GlobalRef,
@@ -89,6 +90,8 @@ struct ExecutionContext {
     pub session_ctx: Arc<SessionContext>,
     /// Whether to enable additional debugging checks & messages
     pub debug_native: bool,
+    /// Whether to write native plans with metrics to stdout
+    pub explain_native: bool,
 }
 
 /// Accept serialized query plan and return the address of the native query plan.
@@ -127,13 +130,24 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_createPlan(
         }
 
         // Whether we've enabled additional debugging on the native side
-        let debug_native = configs
-            .get("debug_native")
-            .and_then(|x| x.parse::<bool>().ok())
-            .unwrap_or(false);
+        let debug_native = parse_bool(&configs, "debug_native")?;
+        let explain_native = parse_bool(&configs, "explain_native")?;
+
+        let worker_threads = configs
+            .get("worker_threads")
+            .map(String::as_str)
+            .unwrap_or("4")
+            .parse::<usize>()?;
+        let blocking_threads = configs
+            .get("blocking_threads")
+            .map(String::as_str)
+            .unwrap_or("10")
+            .parse::<usize>()?;
 
         // Use multi-threaded tokio runtime to prevent blocking spawned tasks if any
         let runtime = tokio::runtime::Builder::new_multi_thread()
+            .worker_threads(worker_threads)
+            .max_blocking_threads(blocking_threads)
             .enable_all()
             .build()?;
 
@@ -169,6 +183,7 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_createPlan(
             metrics,
             session_ctx: Arc::new(session),
             debug_native,
+            explain_native,
         });
 
         Ok(Box::into_raw(exec_context) as i64)
@@ -192,11 +207,7 @@ fn prepare_datafusion_session_context(
 
     // Check if we are using unified memory manager integrated with Spark. Default to false if not
     // set.
-    let use_unified_memory_manager = conf
-        .get("use_unified_memory_manager")
-        .map(String::as_str)
-        .unwrap_or("false")
-        .parse::<bool>()?;
+    let use_unified_memory_manager = parse_bool(conf, "use_unified_memory_manager")?;
 
     if use_unified_memory_manager {
         // Set Comet memory pool for native
@@ -219,7 +230,18 @@ fn prepare_datafusion_session_context(
     // Get Datafusion configuration from Spark Execution context
     // can be configured in Comet Spark JVM using Spark --conf parameters
     // e.g: spark-shell --conf spark.datafusion.sql_parser.parse_float_as_decimal=true
-    let mut session_config = SessionConfig::new().with_batch_size(batch_size);
+    let mut session_config = SessionConfig::new()
+        .with_batch_size(batch_size)
+        // DataFusion partial aggregates can emit duplicate rows so we disable the
+        // skip partial aggregation feature because this is not compatible with Spark's
+        // use of partial aggregates.
+        .set(
+            "datafusion.execution.skip_partial_aggregation_probe_ratio_threshold",
+            // this is the threshold of number of groups / number of rows and the
+            // maximum value is 1.0, so we set the threshold a little higher just
+            // to be safe
+            ScalarValue::Float64(Some(1.1)),
+        );
 
     for (key, value) in conf.iter().filter(|(k, _)| k.starts_with("datafusion.")) {
         session_config = session_config.set_str(key, value);
@@ -227,10 +249,19 @@ fn prepare_datafusion_session_context(
 
     let runtime = RuntimeEnv::new(rt_config).unwrap();
 
-    Ok(SessionContext::new_with_config_rt(
-        session_config,
-        Arc::new(runtime),
-    ))
+    let mut session_ctx = SessionContext::new_with_config_rt(session_config, Arc::new(runtime));
+
+    datafusion_functions_nested::register_all(&mut session_ctx)?;
+
+    Ok(session_ctx)
+}
+
+fn parse_bool(conf: &HashMap<String, String>, name: &str) -> CometResult<bool> {
+    conf.get(name)
+        .map(String::as_str)
+        .unwrap_or("false")
+        .parse::<bool>()
+        .map_err(|e| CometError::Config(format!("Failed to parse boolean config {name}: {e}")))
 }
 
 /// Prepares arrow arrays for output.
@@ -317,17 +348,17 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_executePlan(
         // Because we don't know if input arrays are dictionary-encoded when we create
         // query plan, we need to defer stream initialization to first time execution.
         if exec_context.root_op.is_none() {
-            let planner = PhysicalPlanner::new(exec_context.session_ctx.clone())
+            let planner = PhysicalPlanner::new(Arc::clone(&exec_context.session_ctx))
                 .with_exec_id(exec_context_id);
             let (scans, root_op) = planner.create_plan(
                 &exec_context.spark_plan,
                 &mut exec_context.input_sources.clone(),
             )?;
 
-            exec_context.root_op = Some(root_op.clone());
+            exec_context.root_op = Some(Arc::clone(&root_op));
             exec_context.scans = scans;
 
-            if exec_context.debug_native {
+            if exec_context.explain_native {
                 let formatted_plan_str =
                     DisplayableExecutionPlan::new(root_op.as_ref()).indent(true);
                 info!("Comet native query plan:\n {formatted_plan_str:}");
@@ -360,6 +391,14 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_executePlan(
                     // Update metrics
                     update_metrics(&mut env, exec_context)?;
 
+                    if exec_context.explain_native {
+                        if let Some(plan) = &exec_context.root_op {
+                            let formatted_plan_str =
+                                DisplayableExecutionPlan::with_metrics(plan.as_ref()).indent(true);
+                            info!("Comet native query plan with metrics:\n{formatted_plan_str:}");
+                        }
+                    }
+
                     let long_array = env.new_long_array(1)?;
                     env.set_long_array_region(&long_array, 0, &[-1])?;
 
