@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use arrow::compute::{cast_with_options, CastOptions};
+use futures::{Stream, StreamExt};
 use std::{
     any::Any,
     pin::Pin,
@@ -22,16 +24,16 @@ use std::{
     task::{Context, Poll},
 };
 
-use futures::{Stream, StreamExt};
-
-use arrow_array::{ArrayRef, RecordBatch, RecordBatchOptions};
-use arrow_schema::{DataType, Field, FieldRef, Schema, SchemaRef};
+use arrow_array::{
+    downcast_dictionary_array, make_array, Array, ArrayRef, RecordBatch, RecordBatchOptions,
+};
+use arrow_data::transform::MutableArrayData;
+use arrow_schema::{ArrowError, DataType, Field, FieldRef, Schema, SchemaRef};
 
+use datafusion::physical_plan::metrics::{BaselineMetrics, ExecutionPlanMetricsSet, MetricsSet};
 use datafusion::{execution::TaskContext, physical_expr::*, physical_plan::*};
 use datafusion_common::{arrow_datafusion_err, DataFusionError, Result as DataFusionResult};
 
-use super::copy_or_cast_array;
-
 /// An utility execution node which makes deep copies of input batches.
 ///
 /// In certain scenarios like sort, DF execution nodes only make shallow copy of input batches.
@@ -42,10 +44,21 @@ pub struct CopyExec {
     input: Arc<dyn ExecutionPlan>,
     schema: SchemaRef,
     cache: PlanProperties,
+    metrics: ExecutionPlanMetricsSet,
+    mode: CopyMode,
+}
+
+#[derive(Debug, PartialEq, Clone)]
+pub enum CopyMode {
+    UnpackOrDeepCopy,
+    UnpackOrClone,
 }
 
 impl CopyExec {
-    pub fn new(input: Arc<dyn ExecutionPlan>) -> Self {
+    pub fn new(input: Arc<dyn ExecutionPlan>, mode: CopyMode) -> Self {
+        // change schema to remove dictionary types because CopyExec always unpacks
+        // dictionaries
+
         let fields: Vec<Field> = input
             .schema()
             .fields
@@ -61,7 +74,7 @@ impl CopyExec {
         let schema = Arc::new(Schema::new(fields));
 
         let cache = PlanProperties::new(
-            EquivalenceProperties::new(schema.clone()),
+            EquivalenceProperties::new(Arc::clone(&schema)),
             Partitioning::UnknownPartitioning(1),
             ExecutionMode::Bounded,
         );
@@ -70,6 +83,8 @@ impl CopyExec {
             input,
             schema,
             cache,
+            metrics: ExecutionPlanMetricsSet::default(),
+            mode,
         }
     }
 }
@@ -78,7 +93,7 @@ impl DisplayAs for CopyExec {
     fn fmt_as(&self, t: DisplayFormatType, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match t {
             DisplayFormatType::Default | DisplayFormatType::Verbose => {
-                write!(f, "CopyExec")
+                write!(f, "CopyExec [{:?}]", self.mode)
             }
         }
     }
@@ -90,7 +105,7 @@ impl ExecutionPlan for CopyExec {
     }
 
     fn schema(&self) -> SchemaRef {
-        self.schema.clone()
+        Arc::clone(&self.schema)
     }
 
     fn children(&self) -> Vec<&Arc<dyn ExecutionPlan>> {
@@ -101,12 +116,14 @@ impl ExecutionPlan for CopyExec {
         self: Arc<Self>,
         children: Vec<Arc<dyn ExecutionPlan>>,
     ) -> DataFusionResult<Arc<dyn ExecutionPlan>> {
-        let input = self.input.clone();
+        let input = Arc::clone(&self.input);
         let new_input = input.with_new_children(children)?;
         Ok(Arc::new(CopyExec {
             input: new_input,
-            schema: self.schema.clone(),
+            schema: Arc::clone(&self.schema),
             cache: self.cache.clone(),
+            metrics: self.metrics.clone(),
+            mode: self.mode.clone(),
         }))
     }
 
@@ -116,7 +133,13 @@ impl ExecutionPlan for CopyExec {
         context: Arc<TaskContext>,
     ) -> DataFusionResult<SendableRecordBatchStream> {
         let child_stream = self.input.execute(partition, context)?;
-        Ok(Box::pin(CopyStream::new(self.schema(), child_stream)))
+        Ok(Box::pin(CopyStream::new(
+            self,
+            self.schema(),
+            child_stream,
+            partition,
+            self.mode.clone(),
+        )))
     }
 
     fn statistics(&self) -> DataFusionResult<Statistics> {
@@ -130,33 +153,52 @@ impl ExecutionPlan for CopyExec {
     fn name(&self) -> &str {
         "CopyExec"
     }
+
+    fn metrics(&self) -> Option<MetricsSet> {
+        Some(self.metrics.clone_inner())
+    }
 }
 
 struct CopyStream {
     schema: SchemaRef,
     child_stream: SendableRecordBatchStream,
+    baseline_metrics: BaselineMetrics,
+    mode: CopyMode,
 }
 
 impl CopyStream {
-    fn new(schema: SchemaRef, child_stream: SendableRecordBatchStream) -> Self {
+    fn new(
+        exec: &CopyExec,
+        schema: SchemaRef,
+        child_stream: SendableRecordBatchStream,
+        partition: usize,
+        mode: CopyMode,
+    ) -> Self {
         Self {
             schema,
             child_stream,
+            baseline_metrics: BaselineMetrics::new(&exec.metrics, partition),
+            mode,
         }
     }
 
     // TODO: replace copy_or_cast_array with copy_array if upstream sort kernel fixes
     // dictionary array sorting issue.
     fn copy(&self, batch: RecordBatch) -> DataFusionResult<RecordBatch> {
+        let mut timer = self.baseline_metrics.elapsed_compute().timer();
         let vectors = batch
             .columns()
             .iter()
-            .map(|v| copy_or_cast_array(v))
+            .map(|v| copy_or_unpack_array(v, &self.mode))
             .collect::<Result<Vec<ArrayRef>, _>>()?;
 
         let options = RecordBatchOptions::new().with_row_count(Some(batch.num_rows()));
-        RecordBatch::try_new_with_options(self.schema.clone(), vectors, &options)
-            .map_err(|e| arrow_datafusion_err!(e))
+        let maybe_batch =
+            RecordBatch::try_new_with_options(Arc::clone(&self.schema), vectors, &options)
+                .map_err(|e| arrow_datafusion_err!(e));
+        timer.stop();
+        self.baseline_metrics.record_output(batch.num_rows());
+        maybe_batch
     }
 }
 
@@ -173,6 +215,59 @@ impl Stream for CopyStream {
 
 impl RecordBatchStream for CopyStream {
     fn schema(&self) -> SchemaRef {
-        self.schema.clone()
+        Arc::clone(&self.schema)
+    }
+}
+
+/// Copy an Arrow Array
+fn copy_array(array: &dyn Array) -> ArrayRef {
+    let capacity = array.len();
+    let data = array.to_data();
+
+    let mut mutable = MutableArrayData::new(vec![&data], false, capacity);
+
+    mutable.extend(0, 0, capacity);
+
+    if matches!(array.data_type(), DataType::Dictionary(_, _)) {
+        let copied_dict = make_array(mutable.freeze());
+        let ref_copied_dict = &copied_dict;
+
+        downcast_dictionary_array!(
+            ref_copied_dict => {
+                // Copying dictionary value array
+                let values = ref_copied_dict.values();
+                let data = values.to_data();
+
+                let mut mutable = MutableArrayData::new(vec![&data], false, values.len());
+                mutable.extend(0, 0, values.len());
+
+                let copied_dict = ref_copied_dict.with_values(make_array(mutable.freeze()));
+                Arc::new(copied_dict)
+            }
+            t => unreachable!("Should not reach here: {}", t)
+        )
+    } else {
+        make_array(mutable.freeze())
+    }
+}
+
+/// Copy an Arrow Array or cast to primitive type if it is a dictionary array.
+/// This is used for `CopyExec` to copy/cast the input array. If the input array
+/// is a dictionary array, we will cast the dictionary array to primitive type
+/// (i.e., unpack the dictionary array) and copy the primitive array. If the input
+/// array is a primitive array, we simply copy the array.
+fn copy_or_unpack_array(array: &Arc<dyn Array>, mode: &CopyMode) -> Result<ArrayRef, ArrowError> {
+    match array.data_type() {
+        DataType::Dictionary(_, value_type) => {
+            let options = CastOptions::default();
+            cast_with_options(array, value_type.as_ref(), &options)
+        }
+        _ => {
+            if mode == &CopyMode::UnpackOrDeepCopy {
+                Ok(copy_array(array))
+            } else {
+                Ok(Arc::clone(array))
+            }
+        }
     }
 }
