@@ -38,6 +38,7 @@ use crate::{
     },
     jvm_bridge::{jni_call, JVMClasses},
 };
+use datafusion::physical_plan::metrics::{BaselineMetrics, ExecutionPlanMetricsSet, MetricsSet};
 use datafusion::{
     execution::TaskContext,
     physical_expr::*,
@@ -49,6 +50,8 @@ use jni::{
     sys::jlongArray,
 };
 
+/// ScanExec reads batches of data from Spark via JNI. The source of the scan could be a file
+/// scan or the result of reading a broadcast or shuffle exchange.
 #[derive(Debug, Clone)]
 pub struct ScanExec {
     /// The ID of the execution context that owns this subquery. We use this ID to retrieve the JVM
@@ -56,21 +59,32 @@ pub struct ScanExec {
     pub exec_context_id: i64,
     /// The input source of scan node. It is a global reference of JVM `CometBatchIterator` object.
     pub input_source: Option<Arc<GlobalRef>>,
+    /// A description of the input source for informational purposes
+    pub input_source_description: String,
     /// The data types of columns of the input batch. Converted from Spark schema.
     pub data_types: Vec<DataType>,
     /// The input batch of input data. Used to determine the schema of the input data.
     /// It is also used in unit test to mock the input data from JVM.
     pub batch: Arc<Mutex<Option<InputBatch>>>,
+    /// Cache of expensive-to-compute plan properties
     cache: PlanProperties,
+    /// Metrics collector
+    metrics: ExecutionPlanMetricsSet,
 }
 
 impl ScanExec {
     pub fn new(
         exec_context_id: i64,
         input_source: Option<Arc<GlobalRef>>,
+        input_source_description: &str,
         data_types: Vec<DataType>,
     ) -> Result<Self, CometError> {
         // Scan's schema is determined by the input batch, so we need to set it before execution.
+        // Note that we determine if arrays are dictionary-encoded based on the
+        // first batch. The array may be dictionary-encoded in some batches and not others, and
+        // ScanExec will cast arrays from all future batches to the type determined here, so we
+        // may end up either unpacking dictionary arrays or dictionary-encoding arrays.
+        // Dictionary-encoded primitive arrays are always unpacked.
         let first_batch = if let Some(input_source) = input_source.as_ref() {
             ScanExec::get_next(exec_context_id, input_source.as_obj())?
         } else {
@@ -81,6 +95,8 @@ impl ScanExec {
 
         let cache = PlanProperties::new(
             EquivalenceProperties::new(schema),
+            // The partitioning is not important because we are not using DataFusion's
+            // query planner or optimizer
             Partitioning::UnknownPartitioning(1),
             ExecutionMode::Bounded,
         );
@@ -88,9 +104,11 @@ impl ScanExec {
         Ok(Self {
             exec_context_id,
             input_source,
+            input_source_description: input_source_description.to_string(),
             data_types,
             batch: Arc::new(Mutex::new(Some(first_batch))),
             cache,
+            metrics: ExecutionPlanMetricsSet::default(),
         })
     }
 
@@ -102,11 +120,15 @@ impl ScanExec {
     /// This is necessary since DataFusion doesn't handle dictionary array with values
     /// being primitive type.
     ///
-    /// TODO: revisit this once DF has imprved its dictionary type support. Ideally we shouldn't
+    /// TODO: revisit this once DF has improved its dictionary type support. Ideally we shouldn't
     ///   do this in Comet but rather let DF to handle it for us.
     fn unpack_dictionary_type(dt: &DataType) -> DataType {
         if let DataType::Dictionary(_, vt) = dt {
-            if !matches!(vt.as_ref(), DataType::Utf8 | DataType::Binary) {
+            if !matches!(
+                vt.as_ref(),
+                DataType::Utf8 | DataType::LargeUtf8 | DataType::Binary | DataType::LargeBinary
+            ) {
+                // return the underlying data type
                 return vt.as_ref().clone();
             }
         }
@@ -121,13 +143,12 @@ impl ScanExec {
 
     /// Pull next input batch from JVM.
     pub fn get_next_batch(&mut self) -> Result<(), CometError> {
-        let mut current_batch = self.batch.try_lock().unwrap();
-
         if self.input_source.is_none() {
             // This is a unit test. We don't need to call JNI.
             return Ok(());
         }
 
+        let mut current_batch = self.batch.try_lock().unwrap();
         if current_batch.is_none() {
             let next_batch = ScanExec::get_next(
                 self.exec_context_id,
@@ -146,8 +167,6 @@ impl ScanExec {
             return Ok(InputBatch::EOF);
         }
 
-        let mut env = JVMClasses::get_env()?;
-
         if iter.is_null() {
             return Err(CometError::from(ExecutionError::GeneralError(format!(
                 "Null batch iterator object. Plan id: {}",
@@ -155,6 +174,7 @@ impl ScanExec {
             ))));
         }
 
+        let mut env = JVMClasses::get_env()?;
         let batch_object: JObject = unsafe {
             jni_call!(&mut env,
             comet_batch_iterator(iter).next() -> JObject)?
@@ -171,8 +191,6 @@ impl ScanExec {
 
         let addresses = unsafe { env.get_array_elements(&batch_object, ReleaseMode::NoCopyBack)? };
 
-        let mut inputs: Vec<ArrayRef> = vec![];
-
         // First element is the number of rows.
         let num_rows = unsafe { *addresses.as_ptr() as i64 };
 
@@ -190,9 +208,9 @@ impl ScanExec {
 
         let num_arrays = array_num / 2;
         let array_elements = unsafe { addresses.as_ptr().add(1) };
+        let mut inputs: Vec<ArrayRef> = Vec::with_capacity(num_arrays);
 
-        let mut i: usize = 0;
-        while i < num_arrays {
+        for i in 0..num_arrays {
             let array_ptr = unsafe { *(array_elements.add(i * 2)) };
             let schema_ptr = unsafe { *(array_elements.add(i * 2 + 1)) };
             let array_data = ArrayData::from_spark((array_ptr, schema_ptr))?;
@@ -200,7 +218,6 @@ impl ScanExec {
             // TODO: validate array input data
 
             inputs.push(make_array(array_data));
-            i += 1;
         }
 
         Ok(InputBatch::new(inputs, Some(num_rows as usize)))
@@ -261,10 +278,14 @@ impl ExecutionPlan for ScanExec {
 
     fn execute(
         &self,
-        _: usize,
+        partition: usize,
         _: Arc<TaskContext>,
     ) -> datafusion::common::Result<SendableRecordBatchStream> {
-        Ok(Box::pin(ScanStream::new(self.clone(), self.schema())))
+        Ok(Box::pin(ScanStream::new(
+            self.clone(),
+            self.schema(),
+            partition,
+        )))
     }
 
     fn properties(&self) -> &PlanProperties {
@@ -274,20 +295,24 @@ impl ExecutionPlan for ScanExec {
     fn name(&self) -> &str {
         "ScanExec"
     }
+
+    fn metrics(&self) -> Option<MetricsSet> {
+        Some(self.metrics.clone_inner())
+    }
 }
 
 impl DisplayAs for ScanExec {
     fn fmt_as(&self, t: DisplayFormatType, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match t {
             DisplayFormatType::Default | DisplayFormatType::Verbose => {
-                write!(f, "ScanExec")?;
+                write!(f, "ScanExec: source=[{}], ", self.input_source_description)?;
                 let fields: Vec<String> = self
                     .data_types
                     .iter()
                     .enumerate()
                     .map(|(idx, dt)| format!("col_{idx:}: {dt:}"))
                     .collect();
-                write!(f, ": schema=[{}]", fields.join(", "))?;
+                write!(f, "schema=[{}]", fields.join(", "))?;
             }
         }
         Ok(())
@@ -295,16 +320,26 @@ impl DisplayAs for ScanExec {
 }
 
 /// A async-stream feeds input batch from `Scan` into DataFusion physical plan.
-struct ScanStream {
+struct ScanStream<'a> {
     /// The `Scan` node producing input batches
     scan: ScanExec,
     /// Schema representing the data
     schema: SchemaRef,
+    /// Metrics
+    baseline_metrics: BaselineMetrics,
+    /// Cast options
+    cast_options: CastOptions<'a>,
 }
 
-impl ScanStream {
-    pub fn new(scan: ScanExec, schema: SchemaRef) -> Self {
-        Self { scan, schema }
+impl<'a> ScanStream<'a> {
+    pub fn new(scan: ScanExec, schema: SchemaRef, partition: usize) -> Self {
+        let baseline_metrics = BaselineMetrics::new(&scan.metrics, partition);
+        Self {
+            scan,
+            schema,
+            baseline_metrics,
+            cast_options: CastOptions::default(),
+        }
     }
 
     fn build_record_batch(
@@ -315,55 +350,63 @@ impl ScanStream {
         let schema_fields = self.schema.fields();
         assert_eq!(columns.len(), schema_fields.len());
 
-        // Cast if necessary
-        let cast_options = CastOptions::default();
+        // Cast dictionary-encoded primitive arrays to regular arrays and cast
+        // Utf8/LargeUtf8/Binary arrays to dictionary-encoded if the schema is
+        // defined as dictionary-encoded and the data in this batch is not
+        // dictionary-encoded (could also be the other way around)
         let new_columns: Vec<ArrayRef> = columns
             .iter()
             .zip(schema_fields.iter())
             .map(|(column, f)| {
                 if column.data_type() != f.data_type() {
-                    cast_with_options(column, f.data_type(), &cast_options)
+                    cast_with_options(column, f.data_type(), &self.cast_options)
                 } else {
-                    Ok(column.clone())
+                    Ok(Arc::clone(column))
                 }
             })
             .collect::<Result<Vec<_>, _>>()?;
         let options = RecordBatchOptions::new().with_row_count(Some(num_rows));
-        RecordBatch::try_new_with_options(self.schema.clone(), new_columns, &options)
+        RecordBatch::try_new_with_options(Arc::clone(&self.schema), new_columns, &options)
             .map_err(|e| arrow_datafusion_err!(e))
     }
 }
 
-impl Stream for ScanStream {
+impl<'a> Stream for ScanStream<'a> {
     type Item = DataFusionResult<RecordBatch>;
 
     fn poll_next(self: Pin<&mut Self>, _: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+        let mut timer = self.baseline_metrics.elapsed_compute().timer();
         let mut scan_batch = self.scan.batch.try_lock().unwrap();
-        let input_batch = &*scan_batch;
 
+        let input_batch = &*scan_batch;
         let input_batch = if let Some(batch) = input_batch {
             batch
         } else {
+            timer.stop();
             return Poll::Pending;
         };
 
         let result = match input_batch {
             InputBatch::EOF => Poll::Ready(None),
             InputBatch::Batch(columns, num_rows) => {
-                Poll::Ready(Some(self.build_record_batch(columns, *num_rows)))
+                self.baseline_metrics.record_output(*num_rows);
+                let maybe_batch = self.build_record_batch(columns, *num_rows);
+                Poll::Ready(Some(maybe_batch))
             }
         };
 
         *scan_batch = None;
 
+        timer.stop();
+
         result
     }
 }
 
-impl RecordBatchStream for ScanStream {
+impl<'a> RecordBatchStream for ScanStream<'a> {
     /// Get the schema
     fn schema(&self) -> SchemaRef {
-        self.schema.clone()
+        Arc::clone(&self.schema)
     }
 }
 
