@@ -19,7 +19,11 @@
 #![allow(non_camel_case_types)]
 #![allow(dead_code)]
 #![allow(clippy::upper_case_acronyms)]
-#![allow(clippy::derive_partial_eq_without_eq)] // For prost generated struct
+// For prost generated struct
+#![allow(clippy::derive_partial_eq_without_eq)]
+// The clippy throws an error if the reference clone not wrapped into `Arc::clone`
+// The lint makes easier for code reader/reviewer separate references clones from more heavyweight ones
+#![deny(clippy::clone_on_ref_ptr)]
 
 use jni::{
     objects::{JClass, JString},
