@@ -15,6 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
+// The clippy throws an error if the reference clone not wrapped into `Arc::clone`
+// The lint makes easier for code reader/reviewer separate references clones from more heavyweight ones
+#![deny(clippy::clone_on_ref_ptr)]
+
 // Include generated modules from .proto files.
 #[allow(missing_docs)]
 pub mod spark_expression {
