@@ -22,8 +22,6 @@ package org.apache.comet.exec
 import scala.util.Random
 
 import org.apache.hadoop.fs.Path
-import org.apache.parquet.example.data.simple.SimpleGroup
-import org.apache.parquet.schema.MessageTypeParser
 import org.apache.spark.sql.{CometTestBase, DataFrame, Row}
 import org.apache.spark.sql.catalyst.optimizer.EliminateSorts
 import org.apache.spark.sql.comet.CometHashAggregateExec
@@ -551,7 +549,6 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         Seq(true, false).foreach { dictionaryEnabled =>
           withSQLConf(
             SQLConf.COALESCE_PARTITIONS_ENABLED.key -> "true",
-            CometConf.COMET_SHUFFLE_ENFORCE_MODE_ENABLED.key -> "true",
             CometConf.COMET_BATCH_SIZE.key -> batchSize.toString) {
             withParquetTable(
               (0 until numValues).map(i => (i, Random.nextInt() % numGroups)),
@@ -578,7 +575,6 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         Seq(true, false).foreach { dictionaryEnabled =>
           withSQLConf(
             SQLConf.COALESCE_PARTITIONS_ENABLED.key -> "true",
-            CometConf.COMET_SHUFFLE_ENFORCE_MODE_ENABLED.key -> "true",
             CometConf.COMET_BATCH_SIZE.key -> batchSize.toString) {
             withTempPath { dir =>
               val path = new Path(dir.toURI.toString, "test.parquet")
@@ -619,7 +615,6 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         Seq(true, false).foreach { dictionaryEnabled =>
           withSQLConf(
             SQLConf.COALESCE_PARTITIONS_ENABLED.key -> "true",
-            CometConf.COMET_SHUFFLE_ENFORCE_MODE_ENABLED.key -> "true",
             CometConf.COMET_BATCH_SIZE.key -> batchSize.toString) {
             withTempPath { dir =>
               val path = new Path(dir.toURI.toString, "test.parquet")
@@ -969,7 +964,6 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     withSQLConf(
       SQLConf.COALESCE_PARTITIONS_ENABLED.key -> "true",
       CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
-      CometConf.COMET_SHUFFLE_ENFORCE_MODE_ENABLED.key -> "true",
       CometConf.COMET_SHUFFLE_MODE.key -> "jvm") {
       Seq(true, false).foreach { dictionary =>
         withSQLConf("parquet.enable.dictionary" -> dictionary.toString) {
@@ -1228,7 +1222,9 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("stddev_pop and stddev_samp") {
-    withSQLConf(CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true") {
+    withSQLConf(
+      CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
+      CometConf.COMET_EXPR_STDDEV_ENABLED.key -> "true") {
       Seq("native", "jvm").foreach { cometShuffleMode =>
         withSQLConf(CometConf.COMET_SHUFFLE_MODE.key -> cometShuffleMode) {
           Seq(true, false).foreach { dictionary =>
@@ -1295,90 +1291,4 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     sparkPlan.collect { case s: CometHashAggregateExec => s }.size
   }
 
-  def makeParquetFile(
-      path: Path,
-      total: Int,
-      numGroups: Int,
-      dictionaryEnabled: Boolean): Unit = {
-    val schemaStr =
-      """
-        |message root {
-        |  optional INT32                    _1(INT_8);
-        |  optional INT32                    _2(INT_16);
-        |  optional INT32                    _3;
-        |  optional INT64                    _4;
-        |  optional FLOAT                    _5;
-        |  optional DOUBLE                   _6;
-        |  optional INT32                    _7(DECIMAL(5, 2));
-        |  optional INT64                    _8(DECIMAL(18, 10));
-        |  optional FIXED_LEN_BYTE_ARRAY(16) _9(DECIMAL(38, 37));
-        |  optional INT64                    _10(TIMESTAMP(MILLIS,true));
-        |  optional INT64                    _11(TIMESTAMP(MICROS,true));
-        |  optional INT32                    _12(DATE);
-        |  optional INT32                    _g1(INT_8);
-        |  optional INT32                    _g2(INT_16);
-        |  optional INT32                    _g3;
-        |  optional INT64                    _g4;
-        |  optional FLOAT                    _g5;
-        |  optional DOUBLE                   _g6;
-        |  optional INT32                    _g7(DECIMAL(5, 2));
-        |  optional INT64                    _g8(DECIMAL(18, 10));
-        |  optional FIXED_LEN_BYTE_ARRAY(16) _g9(DECIMAL(38, 37));
-        |  optional INT64                    _g10(TIMESTAMP(MILLIS,true));
-        |  optional INT64                    _g11(TIMESTAMP(MICROS,true));
-        |  optional INT32                    _g12(DATE);
-        |  optional BINARY                   _g13(UTF8);
-        |  optional BINARY                   _g14;
-        |}
-      """.stripMargin
-
-    val schema = MessageTypeParser.parseMessageType(schemaStr)
-    val writer = createParquetWriter(schema, path, dictionaryEnabled = true)
-
-    val rand = scala.util.Random
-    val expected = (0 until total).map { i =>
-      // use a single value for the first page, to make sure dictionary encoding kicks in
-      if (rand.nextBoolean()) None
-      else {
-        if (dictionaryEnabled) Some(i % 10) else Some(i)
-      }
-    }
-
-    expected.foreach { opt =>
-      val record = new SimpleGroup(schema)
-      opt match {
-        case Some(i) =>
-          record.add(0, i.toByte)
-          record.add(1, i.toShort)
-          record.add(2, i)
-          record.add(3, i.toLong)
-          record.add(4, rand.nextFloat())
-          record.add(5, rand.nextDouble())
-          record.add(6, i)
-          record.add(7, i.toLong)
-          record.add(8, (i % 10).toString * 16)
-          record.add(9, i.toLong)
-          record.add(10, i.toLong)
-          record.add(11, i)
-          record.add(12, i.toByte % numGroups)
-          record.add(13, i.toShort % numGroups)
-          record.add(14, i % numGroups)
-          record.add(15, i.toLong % numGroups)
-          record.add(16, rand.nextFloat())
-          record.add(17, rand.nextDouble())
-          record.add(18, i)
-          record.add(19, i.toLong)
-          record.add(20, (i % 10).toString * 16)
-          record.add(21, i.toLong)
-          record.add(22, i.toLong)
-          record.add(23, i)
-          record.add(24, (i % 10).toString * 24)
-          record.add(25, (i % 10).toString * 36)
-        case _ =>
-      }
-      writer.write(record)
-    }
-
-    writer.close()
-  }
 }
