@@ -35,21 +35,22 @@ import org.apache.spark.sql.catalyst.TableIdentifier
 import org.apache.spark.sql.catalyst.catalog.{BucketSpec, CatalogStatistics, CatalogTable}
 import org.apache.spark.sql.catalyst.expressions.Hex
 import org.apache.spark.sql.catalyst.expressions.aggregate.AggregateMode
-import org.apache.spark.sql.comet.{CometBroadcastExchangeExec, CometBroadcastHashJoinExec, CometCollectLimitExec, CometFilterExec, CometHashAggregateExec, CometHashJoinExec, CometProjectExec, CometRowToColumnarExec, CometScanExec, CometSortExec, CometSortMergeJoinExec, CometTakeOrderedAndProjectExec}
+import org.apache.spark.sql.comet.{CometBroadcastExchangeExec, CometBroadcastHashJoinExec, CometCollectLimitExec, CometFilterExec, CometHashAggregateExec, CometHashJoinExec, CometProjectExec, CometScanExec, CometSortExec, CometSortMergeJoinExec, CometSparkToColumnarExec, CometTakeOrderedAndProjectExec}
 import org.apache.spark.sql.comet.execution.shuffle.{CometColumnarShuffle, CometShuffleExchangeExec}
 import org.apache.spark.sql.execution.{CollectLimitExec, ProjectExec, SQLExecution, UnionExec}
+import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanExec
 import org.apache.spark.sql.execution.exchange.{BroadcastExchangeExec, ReusedExchangeExec, ShuffleExchangeExec}
 import org.apache.spark.sql.execution.joins.{BroadcastNestedLoopJoinExec, CartesianProductExec, SortMergeJoinExec}
 import org.apache.spark.sql.execution.reuse.ReuseExchangeAndSubquery
 import org.apache.spark.sql.execution.window.WindowExec
 import org.apache.spark.sql.expressions.Window
-import org.apache.spark.sql.functions.{col, date_add, expr, lead, sum}
+import org.apache.spark.sql.functions.{col, count, date_add, expr, lead, sum}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.internal.SQLConf.SESSION_LOCAL_TIMEZONE
 import org.apache.spark.unsafe.types.UTF8String
 
 import org.apache.comet.CometConf
-import org.apache.comet.CometSparkSessionExtensions.{isSpark33Plus, isSpark34Plus, isSpark40Plus}
+import org.apache.comet.CometSparkSessionExtensions.{isSpark33Plus, isSpark34Plus, isSpark35Plus, isSpark40Plus}
 
 class CometExecSuite extends CometTestBase {
   import testImplicits._
@@ -63,6 +64,112 @@ class CometExecSuite extends CometTestBase {
     }
   }
 
+  test("Sort on single struct should fallback to Spark") {
+    withSQLConf(
+      SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "true",
+      SQLConf.PARQUET_VECTORIZED_READER_ENABLED.key -> "false",
+      CometConf.COMET_EXEC_ENABLED.key -> "true",
+      CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
+      CometConf.COMET_SHUFFLE_MODE.key -> "jvm") {
+      val data1 =
+        Seq(Tuple1(null), Tuple1((1, "a")), Tuple1((2, null)), Tuple1((3, "b")), Tuple1(null))
+
+      withParquetFile(data1) { file =>
+        readParquetFile(file) { df =>
+          val sort = df.sort("_1")
+          checkSparkAnswer(sort)
+        }
+      }
+
+      val data2 =
+        Seq(
+          Tuple2(null, 1),
+          Tuple2((1, "a"), 2),
+          Tuple2((2, null), 3),
+          Tuple2((3, "b"), 5),
+          Tuple2(null, 6))
+
+      withParquetFile(data2) { file =>
+        readParquetFile(file) { df =>
+          val sort = df.sort("_1")
+          checkSparkAnswer(sort)
+        }
+      }
+    }
+  }
+
+  test("Native window operator should be CometUnaryExec") {
+    withTempView("testData") {
+      sql("""
+          |CREATE OR REPLACE TEMPORARY VIEW testData AS SELECT * FROM VALUES
+          |(null, 1L, 1.0D, date("2017-08-01"), timestamp_seconds(1501545600), "a"),
+          |(1, 1L, 1.0D, date("2017-08-01"), timestamp_seconds(1501545600), "a"),
+          |(1, 2L, 2.5D, date("2017-08-02"), timestamp_seconds(1502000000), "a"),
+          |(2, 2147483650L, 100.001D, date("2020-12-31"), timestamp_seconds(1609372800), "a"),
+          |(1, null, 1.0D, date("2017-08-01"), timestamp_seconds(1501545600), "b"),
+          |(2, 3L, 3.3D, date("2017-08-03"), timestamp_seconds(1503000000), "b"),
+          |(3, 2147483650L, 100.001D, date("2020-12-31"), timestamp_seconds(1609372800), "b"),
+          |(null, null, null, null, null, null),
+          |(3, 1L, 1.0D, date("2017-08-01"), timestamp_seconds(1501545600), null)
+          |AS testData(val, val_long, val_double, val_date, val_timestamp, cate)
+          |""".stripMargin)
+      val df = sql("""
+          |SELECT val, cate, count(val) OVER(PARTITION BY cate ORDER BY val ROWS CURRENT ROW)
+          |FROM testData ORDER BY cate, val
+          |""".stripMargin)
+      checkSparkAnswer(df)
+    }
+  }
+
+  test("subquery execution under CometTakeOrderedAndProjectExec should not fail") {
+    assume(isSpark35Plus, "SPARK-45584 is fixed in Spark 3.5+")
+
+    withTable("t1") {
+      sql("""
+            |CREATE TABLE t1 USING PARQUET
+            |AS SELECT * FROM VALUES
+            |(1, "a"),
+            |(2, "a"),
+            |(3, "a") t(id, value)
+            |""".stripMargin)
+      val df = sql("""
+                     |WITH t2 AS (
+                     |  SELECT * FROM t1 ORDER BY id
+                     |)
+                     |SELECT *, (SELECT COUNT(*) FROM t2) FROM t2 LIMIT 10
+                     |""".stripMargin)
+      checkSparkAnswer(df)
+    }
+  }
+
+  test("Window range frame should fall back to Spark") {
+    val df =
+      Seq((1L, "1"), (1L, "1"), (2147483650L, "1"), (3L, "2"), (2L, "1"), (2147483650L, "2"))
+        .toDF("key", "value")
+
+    checkAnswer(
+      df.select(
+        $"key",
+        count("key").over(
+          Window.partitionBy($"value").orderBy($"key").rangeBetween(0, 2147483648L))),
+      Seq(Row(1, 3), Row(1, 3), Row(2, 2), Row(3, 2), Row(2147483650L, 1), Row(2147483650L, 1)))
+    checkAnswer(
+      df.select(
+        $"key",
+        count("key").over(
+          Window.partitionBy($"value").orderBy($"key").rangeBetween(-2147483649L, 0))),
+      Seq(Row(1, 2), Row(1, 2), Row(2, 3), Row(2147483650L, 2), Row(2147483650L, 4), Row(3, 1)))
+  }
+
+  test("Unsupported window expression should fall back to Spark") {
+    checkAnswer(
+      spark.sql("select sum(a) over () from values 1.0, 2.0, 3.0 T(a)"),
+      Row(6.0) :: Row(6.0) :: Row(6.0) :: Nil)
+    checkAnswer(
+      spark.sql("select avg(a) over () from values 1.0, 2.0, 3.0 T(a)"),
+      Row(2.0) :: Row(2.0) :: Row(2.0) :: Nil)
+  }
+
   test("fix CometNativeExec.doCanonicalize for ReusedExchangeExec") {
     assume(isSpark34Plus, "ChunkedByteBuffer is not serializable before Spark 3.4+")
     withSQLConf(
@@ -270,9 +377,7 @@ class CometExecSuite extends CometTestBase {
 
   test("CometExec.executeColumnarCollectIterator can collect ColumnarBatch results") {
     assume(isSpark34Plus, "ChunkedByteBuffer is not serializable before Spark 3.4+")
-    withSQLConf(
-      CometConf.COMET_EXEC_ENABLED.key -> "true",
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true") {
+    withSQLConf(CometConf.COMET_EXEC_ENABLED.key -> "true") {
       withParquetTable((0 until 50).map(i => (i, i + 1)), "tbl") {
         val df = sql("SELECT _1 + 1, _2 + 2 FROM tbl WHERE _1 > 3")
 
@@ -362,9 +467,7 @@ class CometExecSuite extends CometTestBase {
   }
 
   test("Comet native metrics: project and filter") {
-    withSQLConf(
-      CometConf.COMET_EXEC_ENABLED.key -> "true",
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true") {
+    withSQLConf(CometConf.COMET_EXEC_ENABLED.key -> "true") {
       withParquetTable((0 until 5).map(i => (i, i + 1)), "tbl") {
         val df = sql("SELECT _1 + 1, _2 + 2 FROM tbl WHERE _1 > 3")
         df.collect()
@@ -391,7 +494,6 @@ class CometExecSuite extends CometTestBase {
   test("Comet native metrics: SortMergeJoin") {
     withSQLConf(
       CometConf.COMET_EXEC_ENABLED.key -> "true",
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true",
       "spark.sql.adaptive.autoBroadcastJoinThreshold" -> "-1",
       "spark.sql.autoBroadcastJoinThreshold" -> "-1",
       "spark.sql.join.preferSortMergeJoin" -> "true") {
@@ -613,6 +715,20 @@ class CometExecSuite extends CometTestBase {
     }
   }
 
+  test("explain native plan") {
+    // there are no assertions in this test to prove that the explain feature
+    // wrote the expected output to stdout, but we at least test that enabling
+    // the config does not cause any exceptions.
+    withSQLConf(
+      CometConf.COMET_EXPLAIN_NATIVE_ENABLED.key -> "true",
+      SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "-1") {
+      withParquetTable((0 until 5).map(i => (i, i + 1)), "tbl") {
+        val df = sql("select * FROM tbl a join tbl b on a._1 = b._2").select("a._1")
+        checkSparkAnswerAndOperator(df)
+      }
+    }
+  }
+
   test("transformed cometPlan") {
     withParquetTable((0 until 5).map(i => (i, i + 1)), "tbl") {
       val df = sql("select * FROM tbl where _1 >= 2").select("_1")
@@ -1119,7 +1235,7 @@ class CometExecSuite extends CometTestBase {
         .saveAsTable("bucketed_table2")
 
       withSQLConf(
-        "spark.comet.exec.sort_merge_join.disabled" -> "true",
+        CometConf.COMET_EXEC_SORT_MERGE_JOIN_ENABLED.key -> "false",
         SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "0",
         SQLConf.WHOLESTAGE_CODEGEN_ENABLED.key -> "false") {
         val t1 = spark.table("bucketed_table1")
@@ -1227,9 +1343,8 @@ class CometExecSuite extends CometTestBase {
 
   test("disabled/unsupported exec with multiple children should not disappear") {
     withSQLConf(
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "false",
-      CometConf.COMET_EXEC_CONFIG_PREFIX + ".project.enabled" -> "true",
-      CometConf.COMET_EXEC_CONFIG_PREFIX + ".union.enabled" -> "false") {
+      CometConf.COMET_EXEC_PROJECT_ENABLED.key -> "true",
+      CometConf.COMET_EXEC_UNION_ENABLED.key -> "false") {
       withParquetDataFrame((0 until 5).map(Tuple1(_))) { df =>
         val projected = df.selectExpr("_1 as x")
         val unioned = projected.union(df)
@@ -1379,7 +1494,7 @@ class CometExecSuite extends CometTestBase {
     })
   }
 
-  test("RowToColumnar over RangeExec") {
+  test("SparkToColumnar over RangeExec") {
     Seq("true", "false").foreach(aqe => {
       Seq(500, 900).foreach { batchSize =>
         withSQLConf(
@@ -1389,13 +1504,15 @@ class CometExecSuite extends CometTestBase {
           checkSparkAnswerAndOperator(df)
           // empty record batch should also be handled
           val df2 = spark.range(0).selectExpr("id", "id % 8 as k").groupBy("k").sum("id")
-          checkSparkAnswerAndOperator(df2, includeClasses = Seq(classOf[CometRowToColumnarExec]))
+          checkSparkAnswerAndOperator(
+            df2,
+            includeClasses = Seq(classOf[CometSparkToColumnarExec]))
         }
       }
     })
   }
 
-  test("RowToColumnar over RangeExec directly is eliminated for row output") {
+  test("SparkToColumnar over RangeExec directly is eliminated for row output") {
     Seq("true", "false").foreach(aqe => {
       Seq(500, 900).foreach { batchSize =>
         withSQLConf(
@@ -1403,8 +1520,8 @@ class CometExecSuite extends CometTestBase {
           SQLConf.ARROW_EXECUTION_MAX_RECORDS_PER_BATCH.key -> batchSize.toString) {
           val df = spark.range(1000)
           val qe = df.queryExecution
-          qe.executedPlan.collectFirst({ case r: CometRowToColumnarExec => r }) match {
-            case Some(_) => fail("CometRowToColumnarExec should be eliminated")
+          qe.executedPlan.collectFirst({ case r: CometSparkToColumnarExec => r }) match {
+            case Some(_) => fail("CometSparkToColumnarExec should be eliminated")
             case _ =>
           }
         }
@@ -1412,25 +1529,101 @@ class CometExecSuite extends CometTestBase {
     })
   }
 
-  test("RowToColumnar over InMemoryTableScanExec") {
+  test("SparkToColumnar over InMemoryTableScanExec") {
     Seq("true", "false").foreach(aqe => {
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> aqe,
-        CometConf.COMET_SHUFFLE_MODE.key -> "jvm",
-        SQLConf.CACHE_VECTORIZED_READER_ENABLED.key -> "false") {
-        spark
-          .range(1000)
-          .selectExpr("id as key", "id % 8 as value")
-          .toDF("key", "value")
-          .selectExpr("key", "value", "key+1")
-          .createOrReplaceTempView("abc")
-        spark.catalog.cacheTable("abc")
-        val df = spark.sql("SELECT * FROM abc").groupBy("key").count()
-        checkSparkAnswerAndOperator(df, includeClasses = Seq(classOf[CometRowToColumnarExec]))
-      }
+      Seq("true", "false").foreach(cacheVectorized => {
+        withSQLConf(
+          SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> aqe,
+          CometConf.COMET_SHUFFLE_MODE.key -> "jvm",
+          SQLConf.CACHE_VECTORIZED_READER_ENABLED.key -> cacheVectorized) {
+          spark
+            .range(1000)
+            .selectExpr("id as key", "id % 8 as value")
+            .toDF("key", "value")
+            .selectExpr("key", "value", "key+1")
+            .createOrReplaceTempView("abc")
+          spark.catalog.cacheTable("abc")
+          val df = spark.sql("SELECT * FROM abc").groupBy("key").count()
+          checkSparkAnswerAndOperator(df, includeClasses = Seq(classOf[CometSparkToColumnarExec]))
+        }
+      })
     })
   }
 
+  test("SparkToColumnar eliminate redundant in AQE") {
+    withSQLConf(
+      SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "true",
+      CometConf.COMET_SHUFFLE_MODE.key -> "jvm") {
+      val df = spark
+        .range(1000)
+        .selectExpr("id as key", "id % 8 as value")
+        .toDF("key", "value")
+        .groupBy("key")
+        .count()
+      df.collect()
+
+      val planAfter = df.queryExecution.executedPlan
+      assert(planAfter.toString.startsWith("AdaptiveSparkPlan isFinalPlan=true"))
+      val adaptivePlan = planAfter.asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+      val numOperators = adaptivePlan.collect { case c: CometSparkToColumnarExec =>
+        c
+      }
+      assert(numOperators.length == 1)
+    }
+  }
+
+  test("aggregate window function for all types") {
+    val numValues = 2048
+
+    Seq(1, 100, numValues).foreach { numGroups =>
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempPath { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFile(path, numValues, numGroups, dictionaryEnabled)
+          withParquetTable(path.toUri.toString, "tbl") {
+            Seq(128, numValues + 100).foreach { batchSize =>
+              withSQLConf(CometConf.COMET_BATCH_SIZE.key -> batchSize.toString) {
+                (1 to 11).foreach { col =>
+                  val aggregateFunctions =
+                    List(s"COUNT(_$col)", s"MAX(_$col)", s"MIN(_$col)", s"SUM(_$col)")
+                  aggregateFunctions.foreach { function =>
+                    val df1 = sql(s"SELECT $function OVER() FROM tbl")
+                    checkSparkAnswerWithTol(df1, 1e-6)
+
+                    val df2 = sql(s"SELECT $function OVER(order by _2) FROM tbl")
+                    checkSparkAnswerWithTol(df2, 1e-6)
+
+                    val df3 = sql(s"SELECT $function OVER(order by _2 desc) FROM tbl")
+                    checkSparkAnswerWithTol(df3, 1e-6)
+
+                    val df4 = sql(s"SELECT $function OVER(partition by _2 order by _2) FROM tbl")
+                    checkSparkAnswerWithTol(df4, 1e-6)
+                  }
+                }
+
+                // SUM doesn't work for Date type. org.apache.spark.sql.AnalysisException will be thrown.
+                val aggregateFunctionsWithoutSum = List("COUNT(_12)", "MAX(_12)", "MIN(_12)")
+                aggregateFunctionsWithoutSum.foreach { function =>
+                  val df1 = sql(s"SELECT $function OVER() FROM tbl")
+                  checkSparkAnswerWithTol(df1, 1e-6)
+
+                  val df2 = sql(s"SELECT $function OVER(order by _2) FROM tbl")
+                  checkSparkAnswerWithTol(df2, 1e-6)
+
+                  val df3 = sql(s"SELECT $function OVER(order by _2 desc) FROM tbl")
+                  checkSparkAnswerWithTol(df3, 1e-6)
+
+                  val df4 = sql(s"SELECT $function OVER(partition by _2 order by _2) FROM tbl")
+                  checkSparkAnswerWithTol(df4, 1e-6)
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
   test("Windows support") {
     Seq("true", "false").foreach(aqeEnabled =>
       withSQLConf(
@@ -1438,14 +1631,21 @@ class CometExecSuite extends CometTestBase {
         SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> aqeEnabled) {
         withParquetTable((0 until 10).map(i => (i, 10 - i)), "t1") { // TODO: test nulls
           val aggregateFunctions =
-            List("MAX(_1)", "MIN(_1)") // TODO: Test all the aggregates
+            List(
+              "COUNT(_1)",
+              "COUNT(*)",
+              "MAX(_1)",
+              "MIN(_1)",
+              "SUM(_1)"
+            ) // TODO: Test all the aggregates
 
           aggregateFunctions.foreach { function =>
             val queries = Seq(
               s"SELECT $function OVER() FROM t1",
-              s"SELECT $function OVER(order by _2) FROM t1",
-              s"SELECT $function OVER(order by _2 desc) FROM t1",
-              s"SELECT $function OVER(partition by _2 order by _2) FROM t1",
+              // TODO: Range frame is not supported yet.
+              // s"SELECT $function OVER(order by _2) FROM t1",
+              // s"SELECT $function OVER(order by _2 desc) FROM t1",
+              // s"SELECT $function OVER(partition by _2 order by _2) FROM t1",
               s"SELECT $function OVER(rows between 1 preceding and 1 following) FROM t1",
               s"SELECT $function OVER(order by _2 rows between 1 preceding and current row) FROM t1",
               s"SELECT $function OVER(order by _2 rows between current row and 1 following) FROM t1")
@@ -1457,6 +1657,20 @@ class CometExecSuite extends CometTestBase {
         }
       })
   }
+
+  test("read JSON file") {
+    Seq("", "json").foreach { v1List =>
+      withSQLConf(
+        SQLConf.USE_V1_SOURCE_LIST.key -> v1List,
+        CometConf.COMET_EXPLAIN_FALLBACK_ENABLED.key -> "true",
+        CometConf.COMET_CONVERT_FROM_JSON_ENABLED.key -> "true") {
+        spark.read
+          .json("src/test/resources/test-data/json-test-1.ndjson")
+          .createOrReplaceTempView("tbl")
+        checkSparkAnswerAndOperator("SELECT a, b.c, b.d FROM tbl")
+      }
+    }
+  }
 }
 
 case class BucketedTableTestSpec(
