@@ -174,9 +174,7 @@ class CometNativeShuffleSuite extends CometTestBase with AdaptiveSparkPlanHelper
 
   test("fix: Dictionary arrays imported from native should not be overridden") {
     Seq(10, 201).foreach { numPartitions =>
-      withSQLConf(
-        CometConf.COMET_BATCH_SIZE.key -> "10",
-        CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true") {
+      withSQLConf(CometConf.COMET_BATCH_SIZE.key -> "10") {
         withParquetTable((0 until 50).map(i => (1.toString, 2.toString, (i + 1).toLong)), "tbl") {
           val df = sql("SELECT * FROM tbl")
             .filter($"_1" === 1.toString)
