@@ -82,6 +82,44 @@ abstract class ParquetReadSuite extends CometTestBase {
     }
   }
 
+  test("unsupported Spark types") {
+    Seq(
+      NullType -> false,
+      BooleanType -> true,
+      ByteType -> true,
+      ShortType -> true,
+      IntegerType -> true,
+      LongType -> true,
+      FloatType -> true,
+      DoubleType -> true,
+      BinaryType -> true,
+      StringType -> true,
+      ArrayType(TimestampType) -> false,
+      StructType(
+        Seq(
+          StructField("f1", DecimalType.SYSTEM_DEFAULT),
+          StructField("f2", StringType))) -> false,
+      MapType(keyType = LongType, valueType = DateType) -> false,
+      StructType(Seq(StructField("f1", ByteType), StructField("f2", StringType))) -> false,
+      MapType(keyType = IntegerType, valueType = BinaryType) -> false).foreach {
+      case (dt, expected) =>
+        assert(CometScanExec.isTypeSupported(dt) == expected)
+        assert(CometBatchScanExec.isTypeSupported(dt) == expected)
+    }
+  }
+
+  test("unsupported Spark schema") {
+    Seq(
+      Seq(StructField("f1", IntegerType), StructField("f2", BooleanType)) -> true,
+      Seq(StructField("f1", IntegerType), StructField("f2", ArrayType(IntegerType))) -> false,
+      Seq(
+        StructField("f1", MapType(keyType = LongType, valueType = StringType)),
+        StructField("f2", ArrayType(DoubleType))) -> false).foreach { case (schema, expected) =>
+      assert(CometScanExec.isSchemaSupported(StructType(schema)) == expected)
+      assert(CometBatchScanExec.isSchemaSupported(StructType(schema)) == expected)
+    }
+  }
+
   test("simple count") {
     withParquetTable((0 until 10).map(i => (i, i.toString)), "tbl") {
       assert(sql("SELECT * FROM tbl WHERE _1 % 2 == 0").count() == 5)
@@ -621,8 +659,7 @@ abstract class ParquetReadSuite extends CometTestBase {
     Seq(false, true).foreach { enableDictionary =>
       withSQLConf(
         CometConf.COMET_BATCH_SIZE.key -> 7.toString,
-        CometConf.COMET_EXEC_ENABLED.key -> "true",
-        CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true") {
+        CometConf.COMET_EXEC_ENABLED.key -> "true") {
         // Make sure this works with Comet native execution too
         val data = (1 to 100)
           .map(_ % 5) // trigger dictionary encoding
@@ -709,7 +746,6 @@ abstract class ParquetReadSuite extends CometTestBase {
     withSQLConf(
       CometConf.COMET_BATCH_SIZE.key -> 7.toString,
       CometConf.COMET_EXEC_ENABLED.key -> "false",
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> "true",
       CometConf.COMET_ENABLED.key -> "true") {
       Seq("a", null).foreach { partValue =>
         withTempPath { dir =>
@@ -1248,7 +1284,6 @@ abstract class ParquetReadSuite extends CometTestBase {
     withSQLConf(
       CometConf.COMET_ENABLED.key -> cometEnabled,
       CometConf.COMET_EXEC_ENABLED.key -> cometEnabled,
-      CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key -> cometEnabled,
       SQLConf.USE_V1_SOURCE_LIST.key -> v1.getOrElse("")) {
       withParquetTable(Seq((Long.MaxValue, 1), (Long.MaxValue, 2)), "tbl") {
         val df = spark.sql("select * from tbl")
