@@ -36,7 +36,7 @@ import org.apache.parquet.hadoop.example.ExampleParquetWriter
 import org.apache.parquet.schema.{MessageType, MessageTypeParser}
 import org.apache.spark._
 import org.apache.spark.internal.config.{MEMORY_OFFHEAP_ENABLED, MEMORY_OFFHEAP_SIZE, SHUFFLE_MANAGER}
-import org.apache.spark.sql.comet.{CometBatchScanExec, CometBroadcastExchangeExec, CometExec, CometRowToColumnarExec, CometScanExec, CometScanWrapper, CometSinkPlaceHolder}
+import org.apache.spark.sql.comet.{CometBatchScanExec, CometBroadcastExchangeExec, CometExec, CometScanExec, CometScanWrapper, CometSinkPlaceHolder, CometSparkToColumnarExec}
 import org.apache.spark.sql.comet.execution.shuffle.{CometColumnarShuffle, CometNativeShuffle, CometShuffleExchangeExec}
 import org.apache.spark.sql.execution.{ColumnarToRowExec, ExtendedMode, InputAdapter, SparkPlan, WholeStageCodegenExec}
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
@@ -78,8 +78,7 @@ abstract class CometTestBase
     conf.set(SQLConf.COALESCE_PARTITIONS_ENABLED.key, "false")
     conf.set(CometConf.COMET_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_ENABLED.key, "true")
-    conf.set(CometConf.COMET_EXEC_ALL_OPERATOR_ENABLED.key, "true")
-    conf.set(CometConf.COMET_ROW_TO_COLUMNAR_ENABLED.key, "true")
+    conf.set(CometConf.COMET_SPARK_TO_ARROW_ENABLED.key, "true")
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "2g")
     conf
   }
@@ -170,11 +169,11 @@ abstract class CometTestBase
   }
 
   protected def checkCometOperators(plan: SparkPlan, excludedClasses: Class[_]*): Unit = {
-    val wrapped = wrapCometRowToColumnar(plan)
+    val wrapped = wrapCometSparkToColumnar(plan)
     wrapped.foreach {
       case _: CometScanExec | _: CometBatchScanExec =>
       case _: CometSinkPlaceHolder | _: CometScanWrapper =>
-      case _: CometRowToColumnarExec =>
+      case _: CometSparkToColumnarExec =>
       case _: CometExec | _: CometShuffleExchangeExec =>
       case _: CometBroadcastExchangeExec =>
       case _: WholeStageCodegenExec | _: ColumnarToRowExec | _: InputAdapter =>
@@ -200,10 +199,10 @@ abstract class CometTestBase
   }
 
   /** Wraps the CometRowToColumn as ScanWrapper, so the child operators will not be checked */
-  private def wrapCometRowToColumnar(plan: SparkPlan): SparkPlan = {
+  private def wrapCometSparkToColumnar(plan: SparkPlan): SparkPlan = {
     plan.transformDown {
       // don't care the native operators
-      case p: CometRowToColumnarExec => CometScanWrapper(null, p)
+      case p: CometSparkToColumnarExec => CometScanWrapper(null, p)
     }
   }
 
@@ -816,4 +815,91 @@ abstract class CometTestBase
       vertical: Boolean = false): String = {
     df.showString(_numRows, truncate, vertical)
   }
+
+  def makeParquetFile(
+      path: Path,
+      total: Int,
+      numGroups: Int,
+      dictionaryEnabled: Boolean): Unit = {
+    val schemaStr =
+      """
+        |message root {
+        |  optional INT32                    _1(INT_8);
+        |  optional INT32                    _2(INT_16);
+        |  optional INT32                    _3;
+        |  optional INT64                    _4;
+        |  optional FLOAT                    _5;
+        |  optional DOUBLE                   _6;
+        |  optional INT32                    _7(DECIMAL(5, 2));
+        |  optional INT64                    _8(DECIMAL(18, 10));
+        |  optional FIXED_LEN_BYTE_ARRAY(16) _9(DECIMAL(38, 37));
+        |  optional INT64                    _10(TIMESTAMP(MILLIS,true));
+        |  optional INT64                    _11(TIMESTAMP(MICROS,true));
+        |  optional INT32                    _12(DATE);
+        |  optional INT32                    _g1(INT_8);
+        |  optional INT32                    _g2(INT_16);
+        |  optional INT32                    _g3;
+        |  optional INT64                    _g4;
+        |  optional FLOAT                    _g5;
+        |  optional DOUBLE                   _g6;
+        |  optional INT32                    _g7(DECIMAL(5, 2));
+        |  optional INT64                    _g8(DECIMAL(18, 10));
+        |  optional FIXED_LEN_BYTE_ARRAY(16) _g9(DECIMAL(38, 37));
+        |  optional INT64                    _g10(TIMESTAMP(MILLIS,true));
+        |  optional INT64                    _g11(TIMESTAMP(MICROS,true));
+        |  optional INT32                    _g12(DATE);
+        |  optional BINARY                   _g13(UTF8);
+        |  optional BINARY                   _g14;
+        |}
+      """.stripMargin
+
+    val schema = MessageTypeParser.parseMessageType(schemaStr)
+    val writer = createParquetWriter(schema, path, dictionaryEnabled = true)
+
+    val rand = scala.util.Random
+    val expected = (0 until total).map { i =>
+      // use a single value for the first page, to make sure dictionary encoding kicks in
+      if (rand.nextBoolean()) None
+      else {
+        if (dictionaryEnabled) Some(i % 10) else Some(i)
+      }
+    }
+
+    expected.foreach { opt =>
+      val record = new SimpleGroup(schema)
+      opt match {
+        case Some(i) =>
+          record.add(0, i.toByte)
+          record.add(1, i.toShort)
+          record.add(2, i)
+          record.add(3, i.toLong)
+          record.add(4, rand.nextFloat())
+          record.add(5, rand.nextDouble())
+          record.add(6, i)
+          record.add(7, i.toLong)
+          record.add(8, (i % 10).toString * 16)
+          record.add(9, i.toLong)
+          record.add(10, i.toLong)
+          record.add(11, i)
+          record.add(12, i.toByte % numGroups)
+          record.add(13, i.toShort % numGroups)
+          record.add(14, i % numGroups)
+          record.add(15, i.toLong % numGroups)
+          record.add(16, rand.nextFloat())
+          record.add(17, rand.nextDouble())
+          record.add(18, i)
+          record.add(19, i.toLong)
+          record.add(20, (i % 10).toString * 16)
+          record.add(21, i.toLong)
+          record.add(22, i.toLong)
+          record.add(23, i)
+          record.add(24, (i % 10).toString * 24)
+          record.add(25, (i % 10).toString * 36)
+        case _ =>
+      }
+      writer.write(record)
+    }
+
+    writer.close()
+  }
 }
